/**
 * This code is free to distribute and use but may not be sold or used in any other way to make money
 * without prior written consent of Jason McDonald, www.mcdonaldland.info. 
 * 
 * Copyright 2007, Jason McDonald
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace ImageOrganizer
{

    public partial class frmMain : Form
    {
        private const int FO_DELETE = 3;
        private const int FOF_ALLOWUNDO = 0x40;
        private const int FOF_NOCONFIRMATION = 0x0010;

        [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto, Pack=1)]
        public struct SHFILEOPSTRUCT
        {
	        public IntPtr hwnd;
	        [MarshalAs(UnmanagedType.U4)] public int wFunc;
	        public string pFrom;
	        public string pTo;
	        public short fFlags;
	        [MarshalAs(UnmanagedType.Bool)] public bool fAnyOperationsAborted;
	        public IntPtr hNameMappings;
	        public string lpszProgressTitle;
        }

        [DllImport("shell32.dll", CharSet=CharSet.Auto)]
        static extern int SHFileOperation(ref SHFILEOPSTRUCT FileOp);





        ASCIIEncoding encoding = new ASCIIEncoding();
        Boolean stopProcessing = false;
        Hashtable imgs = new Hashtable();
        Hashtable dups = new Hashtable();

        private void recycle(FileInfo file) {

            SHFILEOPSTRUCT fileop = new SHFILEOPSTRUCT();
            fileop.wFunc = FO_DELETE;
            fileop.pFrom = file.FullName + '\0' + '\0';
            fileop.fFlags = FOF_ALLOWUNDO | FOF_NOCONFIRMATION;

            SHFileOperation(ref fileop);
        }

        public frmMain()
        {
            InitializeComponent();
        }

        private void btnGo_Click(object sender, EventArgs e)
        {
            if (isEmpty(txtImgDir.Text))
            {
                MessageBox.Show("Please enter a directory where images can be found.", "Missing Required Fields");
                return;
            }

            DirectoryInfo imgDir = new DirectoryInfo(txtImgDir.Text);
            if (imgDir == null)
            {
                MessageBox.Show("The directory entered to find pictures in could not be found.", "Invalid Data Supplied");
                return;
            }

            cleanUp();

            toggleAvailability();
            processImages(imgDir);
            toggleAvailability();

            if (stopProcessing)
            {
                lblMsg.Text = "Processing stopped.";
            }
        }

        private void cleanUp()
        {
            imgs.Clear();
            dups.Clear();
            lblMsg.Text = "";
            stopProcessing = false;
            tvDups.Nodes.Clear();
            pic.Image = null;
        }

        private void processImages(DirectoryInfo imgDir)
        {
            Application.DoEvents();

            if (stopProcessing)
            {
                return;
            }

            DirectoryInfo[] dirs = imgDir.GetDirectories();
            foreach (DirectoryInfo dir in dirs) 
            {
                Application.DoEvents();
                processImages(dir);
            }
            
            FileInfo[] files = imgDir.GetFiles();
            foreach (FileInfo file in files)
            {
                Application.DoEvents();

                if (stopProcessing)
                {
                    return;
                }

                processImage(file);
            }
        }

        private void processImage(FileInfo file)
        {
            Image img;

            try
            {
                img = Image.FromFile(file.FullName);
            } catch (Exception e) {
                return; 
            }

            foreach (PropertyItem pi in img.PropertyItems)
            {
                Application.DoEvents();
                if (pi.Id == 0x0132)
                {
                    String timestamp = encoding.GetString(pi.Value).Trim();

                    if (imgs.ContainsKey(timestamp))
                    {
                        // An image with this time stamp already exists - add this 
                        // to the list of potential duplicates under this key.
                        
                        // First let's see if the image in this key has already been 
                        // added - if not we will need to add it before adding the
                        // file currently being processed.
                        FileInfo f = (FileInfo) imgs[timestamp];

                        if (f != null)
                        {
                            f = (FileInfo) imgs[timestamp];

                            tvDups.Nodes.Add(timestamp, timestamp, 0, 0);
                            tvDups.Nodes[timestamp].Nodes.Add(f.FullName, f.FullName, 1, 2);
                            dups.Add(timestamp + f.Name, f.FullName);
                            imgs[timestamp] = null;
                        }

                        // Add the file we are processing right now
                        int cnt = 0;
                        String fn = file.Name;

                        while (dups.ContainsKey(timestamp + fn + (cnt > 0 ? " " + cnt : "")))
                        {
                            cnt++;
                        }

                        if (cnt > 0)
                        {
                            fn += " " + cnt;
                        }

                        tvDups.Nodes[timestamp].Nodes.Add(fn, file.FullName, 1, 2);
                        dups.Add(timestamp + fn, file.FullName);
                    }
                    else
                    {
                        imgs.Add(timestamp, file);
                    }

                    break;
                }
            }
        }

        private void toggleAvailability()
        {
            txtImgDir.Enabled = !txtImgDir.Enabled;
            btnBrowseImg.Enabled = !btnBrowseImg.Enabled;
            btnGo.Enabled = !btnGo.Enabled;
            btnStop.Enabled = !btnStop.Enabled;
        }

        private Boolean isEmpty(String str)
        {
            return String.IsNullOrEmpty(str);
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            stopProcessing = true;
        }

        private void btnBrowseImg_Click(object sender, EventArgs e)
        {
            browse(txtImgDir);
        }


        private void browse(TextBox txt)
        {
            folderBrowserDialog1.ShowDialog(this);
            txt.Text = folderBrowserDialog1.SelectedPath;
        }

        private void tvDups_AfterSelect(object sender, TreeViewEventArgs e)
        {
            TreeNode node = tvDups.SelectedNode;
            if (node.Nodes.Count > 0)
            {
                lblSelectedNode.Text = tvDups.SelectedNode.Text + " (" + node.Nodes.Count + " images)";
            }
            else
            {
                String filePath = (String) dups[tvDups.SelectedNode.Parent.Text + tvDups.SelectedNode.Text];
                lblSelectedNode.Text = filePath;
                pic.Load(tvDups.SelectedNode.Text);
            }
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            String filePath = tvDups.SelectedNode.Text;

            if (DialogResult.Yes == MessageBox.Show(this, "Are you sure you want to delete this image?\n\n" + filePath,
                    "Confirm Delete", MessageBoxButtons.YesNo))
            {
                FileInfo file = new FileInfo(filePath);
                recycle(file);
                tvDups.Nodes.Remove(tvDups.SelectedNode);
            }            
        }
    }
}